"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
function randomString() {
    return Math.random()
        .toString(36)
        .substring(2, 10);
}
class TypeError extends Error {
    constructor(message, cause) {
        super(message);
        this.cause = cause;
    }
}
exports.TypeError = TypeError;
class TypeConf {
    constructor() {
        this.override = {};
        this.stores = {};
        this.rootAccessor = util_1.baseAccessor;
    }
    resolve(name) {
        if (name in this.override) {
            return this.override[name];
        }
        return this.rootAccessor(name, false);
    }
    exists(name) {
        if (name in this.override) {
            return true;
        }
        return this.rootAccessor(name, true);
    }
    addStore(store, name) {
        this.stores[name] = store;
        this.rootAccessor = util_1.createAccessor(store, this.rootAccessor);
    }
    withStore(storage, name = randomString()) {
        const store = util_1.createStore(Object.assign({}, storage));
        this.addStore(store, name);
        return this;
    }
    withSupplier(supplier, name = randomString()) {
        const store = util_1.createStore(supplier);
        this.addStore(store, name);
        return this;
    }
    set(key, value) {
        this.override[key] = value;
        return this;
    }
    unset(key) {
        delete this.override[key];
        return this;
    }
    get(name, transform) {
        if (typeof transform === 'function') {
            return transform(this.resolve(name));
        }
        else {
            return this.resolve(name);
        }
    }
    getString(name, fallback) {
        const value = this.resolve(name);
        if (typeof value === 'string') {
            return value;
        }
        if (value !== undefined) {
            return JSON.stringify(value);
        }
        if (typeof fallback === 'string') {
            return fallback;
        }
        if (fallback === undefined) {
            return;
        }
        throw new TypeError(`Not a string: ${fallback}`);
    }
    getNumber(name, fallback) {
        const value = this.resolve(name);
        if (typeof value === 'number' && !isNaN(value)) {
            return value;
        }
        if (typeof value === 'string') {
            const parsedValue = parseFloat(value);
            if (!isNaN(parsedValue)) {
                return parsedValue;
            }
            throw new TypeError(`Not a number: ${value}`);
        }
        if (value !== undefined && fallback === undefined) {
            throw new TypeError(`Not a number: ${value}`);
        }
        if (typeof fallback === 'number' && !isNaN(fallback)) {
            return fallback;
        }
        if (fallback === undefined) {
            return;
        }
        throw new TypeError(`Not a number: ${fallback}`);
    }
    getBoolean(name) {
        const value = this.resolve(name);
        return value !== false && value !== 'false' && this.exists(name);
    }
    getObject(name, fallback) {
        const value = this.resolve(name);
        if (typeof value === 'object' && value) {
            return value;
        }
        if (typeof value === 'string' && value) {
            try {
                return JSON.parse(value);
            }
            catch (e) {
                throw new TypeError(`Not an object: ${value}`);
            }
        }
        if (fallback === undefined) {
            return;
        }
        if (typeof fallback === 'object') {
            return fallback;
        }
        throw new TypeError(`Not an object: ${fallback}`);
    }
    getType(name, newable, fallback) {
        const value = this.resolve(name);
        if (value === undefined) {
            return fallback;
        }
        try {
            return new newable(value);
        }
        catch (e) {
            throw new TypeError(`Cannot instantiate ${newable.name} from ${value}.`, e);
        }
    }
}
exports.default = TypeConf;
//# sourceMappingURL=TypeConf.js.map